//direct3d graphics code

//Includes
//DirectX headers
#include <d3d9.h>
#include <d3dx9.h>
//Regular headers
#include <ostream>
#include <iostream>
//Created Headers
#include "dxgraphics.h"

//namespaces
using namespace std;

//variables
LPDIRECT3D9 d3d = NULL;//main d3d object
LPDIRECT3DDEVICE9 d3ddev = NULL;//main d3d device
LPDIRECT3DSURFACE9 backbuffer = NULL;//backbuffer surface
LPD3DXSPRITE sprite_handler;

//initializes direct3d to the given specifications
int Init_Direct3D (HWND hwnd, int width, int height)
{
	HRESULT result;

	//initialize Direct3D
	d3d = Direct3DCreate9(D3D_SDK_VERSION);
	if(d3d == NULL)
	{
		cerr<<"Error creating Direct3D object\n";
		return 0;
	}

	//set d3d presentation params
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp , sizeof(d3dpp));

	d3dpp.Windowed= FALSE;
	d3dpp.SwapEffect=D3DSWAPEFFECT_COPY;
	d3dpp.BackBufferFormat=D3DFMT_X8R8G8B8;
	d3dpp.BackBufferCount=1;
	d3dpp.BackBufferWidth=width;
	d3dpp.BackBufferHeight=height;
	d3dpp.hDeviceWindow=hwnd;
	d3dpp.EnableAutoDepthStencil=TRUE;
	d3dpp.AutoDepthStencilFormat=D3DFMT_D16;
	d3dpp.PresentationInterval=D3DPRESENT_INTERVAL_IMMEDIATE;

	//create d3d device and fill with d3d parameter table (d3dpp)
	result=d3d->CreateDevice(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,hwnd,D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp,&d3ddev);
	if(result != D3D_OK)
	{
		cerr<<"Error creating Direct3D device\n";
		return 0;
	}

	//clear backbuffer to black
	d3ddev->Clear(0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,0),1.0f,0);

	//get pointer to back buffer
	d3ddev->GetBackBuffer(0,0,D3DBACKBUFFER_TYPE_MONO,&backbuffer);

	//create sprite handler
	result=D3DXCreateSprite(d3ddev, &sprite_handler);
	if(result!=D3D_OK)
	{
		cerr<<"Error Creating sprite handler\n";
		return 0;
	}
	cerr<<"Sprite Handler Created\n";

	cerr<<"Direct3D objects initialized\n";
	return 1;
}

//returns a surface loaded with an image from the given filename
//transcolor 0 for no transparency masking else use d3dcolor_xrgb() to render as transparent color
//as black
LPDIRECT3DSURFACE9 LoadSurface(char *filename, D3DCOLOR transcolor)
{
	//temporary surface to be returned
	LPDIRECT3DSURFACE9 image= NULL;
	//holds info about image
	D3DXIMAGE_INFO info;

	HRESULT result;

	//get bitmap properties from file
	result=D3DXGetImageInfoFromFile(filename,&info);
	if(result !=D3D_OK)
	{
		cerr<<"D3DXGetImageInfoFromFile Failed\n";
		return NULL;}

	//create a new surface
	result = d3ddev->CreateOffscreenPlainSurface(info.Width,info.Height,
		D3DFMT_X8R8G8B8,D3DPOOL_DEFAULT,
		&image,NULL);
	if(result!=D3D_OK)
	{
		cerr<<"CreateOffscreenPlainSurface failed\n";
		return NULL;}

	//load surface from file into newly created surface
	result= D3DXLoadSurfaceFromFile( image,NULL,NULL,filename,NULL,D3DX_DEFAULT,transcolor,NULL);
	if(result!=D3D_OK)
	{
		cerr<<"D3DXLoadSurfaceFromFile failed\n";
		return NULL;}

	cerr<<"LoadSurface() successful\n";
	return image;
}

LPDIRECT3DTEXTURE9 LoadTexture(char *filename, D3DCOLOR transcolor)
{
	//texture
	LPDIRECT3DTEXTURE9 texture=NULL;

	//struct for image info
	D3DXIMAGE_INFO info;

	HRESULT result;

	//get width and height from file
	result=D3DXGetImageInfoFromFile(filename, &info);
	if(result!=D3D_OK)
	{
		cerr<<"D3DXGetImageInfoFromFile failed\n";
		return NULL;
	}

	//create new texture from file
	D3DXCreateTextureFromFileEx(d3ddev,filename,info.Width,info.Height,1,D3DPOOL_DEFAULT,
		D3DFMT_UNKNOWN,D3DPOOL_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,transcolor,&info,NULL,
		&texture);
	if(result!=D3D_OK)
	{
		cerr<<"CreateTextureFromFileEx failed\n";
		return NULL;
	}
	
	cerr<<"LoadTexture() Successful\n";

	return texture;
}
