//Main Game Source
//Kid Kombat
//Programming by Adam Sepanski for the Charlotte Game Development Group
//Created on 3/26/06, Version 1.0 Released 8/19/06

//Includes
//regular headers
#include <ostream>
#include <iostream>
//Created Headers
#include "Game.h"

//namespaces
using namespace std;

//constants
const int PLAYER_VELOCITY = 3;//players movement speed
const int AIRPLANE_X_VELOCITY = 8;//airplane's x movement speed
const int AIRPLANE_Y_VELOCITY = 6;//airplane's y movement speed
const int AIRPLANE_LIFETIME = 170;//airplane's alive time
const int NUM_OF_DESKS=4;//number of desks
const int KID_HORIZONTAL_BOUNDING_REDUCTION=20;//pixel reduction for bounding box (used for desk and airplane collisions)
const int KID_VERTICAL_BOUNDING_REDUCTION=24;//pixel reduction for bounding box (used only for airplane collisions)
const int WINNING_CONDITION=5;//how many points it takes to win

//Game States
const int GAME_RUNNING=1; //Game is being played
const int GAME_MENU=2; //game is in main menu
const int GAME_RIGHT_PLAYER_SCORE=3; //right player scored
const int GAME_LEFT_PLAYER_SCORE=4; //left player scored
const int GAME_RIGHT_PLAYER_WINS=6;//right player wins the game
const int GAME_LEFT_PLAYER_WINS=7;//left player wins the game
const int GAME_RESET=5; //reset positions and variables
const int GAME_MENU_CREDITS=8;//credits
const int GAME_MENU_HELP=9;//help

//Surfaces
LPDIRECT3DSURFACE9 background_surface;//background surface
LPDIRECT3DSURFACE9 mainmenu_surface;//main menu surface

//Sprites
SPRITE KidRight;//Right Player
SPRITE KidLeft;//Left Player
SPRITE AirplaneLeft;//left player's projectile
SPRITE AirplaneRight;//right player's projectile
SPRITE Desk[NUM_OF_DESKS];
SPRITE RightPlayerScores,LeftPlayerScores;//background image for when right and left player score
SPRITE RightPlayerName,LeftPlayerName;//sprites for chalkboard name
SPRITE RightPlayerChalkboardScore,LeftPlayerChalkboardScore;//sprites for chalkboard score
SPRITE RightPlayerWins,LeftPlayerWins;//sprites for game winning condition
SPRITE MenuSelector;//sprite for main menu selection
SPRITE CreditMenu;//sprite for credits
SPRITE HelpMenu;//sprite for help

//Sounds
//CSound *sound_test;//for testing purposes
CSound *sound_hit1,*sound_hit2,*sound_hit3;//sound when player is hit

//Timing variable
long start=GetTickCount();//for timing
long counter=0;//for testing frame count

//Keyboard variables
//these are necessary to detect a key press not key down
bool RightKeyPressed=false;
bool LeftKeyPressed=false;
bool UpKeyPressed=false;
bool DownKeyPressed=false;
bool AKeyPressed=false;
bool SKeyPressed=false;
bool DKeyPressed=false;
bool WKeyPressed=false;
bool RcontrolKeyPressed=false;
bool FKeyPressed=false;
bool EscKeyPressed=false;
bool EnterKeyPressed=false;

//Misc.
D3DXVECTOR3 position(0,0,0);//vector that holds sprite position for sprite handler
int GameState=0;//holds the current game status
int LeftPlayerScore=0,RightPlayerScore=0;//holds left and right player score
int MenuSelectorPosition=1;//menu position

//////////////////////////////////////
////////////Functions/////////////////

//initializes game
int Game_Init(HWND hwnd)
{

	//random number seed
	srand(time(NULL));

	//load background images onto surfaces
	background_surface=LoadSurface("classroom.bmp",0);
	mainmenu_surface=LoadSurface("ssfrontmenu.jpg",0);

	//load sprite image onto texture, with pink as transparent
	KidRight.texture=LoadTexture("KidRight.bmp",D3DCOLOR_XRGB(255,0,255));

	//load left player image onto texture
	KidLeft.texture=LoadTexture("kidleft.bmp",D3DCOLOR_XRGB(255,0,255));

	//load left projectile image onto texture
	AirplaneLeft.texture=LoadTexture("airplane.bmp",D3DCOLOR_XRGB(255,0,255));

	//load right projectile image onto texture
	AirplaneRight.texture=LoadTexture("airplane.bmp",D3DCOLOR_XRGB(255,0,255));

	//load desk image onto texture
	for(int index=0;index<NUM_OF_DESKS;index++)
	{
		Desk[index].texture=LoadTexture("desk.bmp",D3DCOLOR_XRGB(255,0,255));
	}

	//load right and left player scoring image
	RightPlayerScores.texture=LoadTexture("Rightplayerscores.bmp",D3DCOLOR_XRGB(255,0,255));
	LeftPlayerScores.texture=LoadTexture("Leftplayerscores.bmp",D3DCOLOR_XRGB(255,0,255));
	RightPlayerName.texture=LoadTexture("rightplayername.bmp",D3DCOLOR_XRGB(255,0,255));
	LeftPlayerName.texture=LoadTexture("leftplayername.bmp",D3DCOLOR_XRGB(255,0,255));
	RightPlayerChalkboardScore.texture=LoadTexture("scores.bmp",D3DCOLOR_XRGB(255,0,255));
	LeftPlayerChalkboardScore.texture=LoadTexture("scores.bmp",D3DCOLOR_XRGB(255,0,255));
	//winning image
	RightPlayerWins.texture=LoadTexture("RightPlayerWins.bmp",D3DCOLOR_XRGB(255,0,255));
	LeftPlayerWins.texture=LoadTexture("LeftPlayerWins.bmp",D3DCOLOR_XRGB(255,0,255));

	//load menu selector
	MenuSelector.texture=LoadTexture("airplanecursor.png",D3DCOLOR_XRGB(255,0,255));
	//load credits
	CreditMenu.texture=LoadTexture("credits.jpg",D3DCOLOR_XRGB(255,0,255));
	//load help
	HelpMenu.texture=LoadTexture("helpmenu.jpg",D3DCOLOR_XRGB(255,0,255));


	//set Right Kid properties
	KidRight.x=750;
	KidRight.y=300;
	KidRight.width=56;
	KidRight.height=64;
	KidRight.vx=0;
	KidRight.vy=0;
	KidRight.direction=WEST;
	KidRight.animdelay=80;
	KidRight.animcount=0;
	KidRight.rectsource.top=0;
	KidRight.rectsource.bottom=KidRight.height;
	KidRight.rectsource.left=0;
	KidRight.rectsource.right=KidRight.width;
	KidRight.visible=true;

	//set Left Kid Properties
	KidLeft.x=50;
	KidLeft.y=300;
	KidLeft.width=56;
	KidLeft.height=64;
	KidLeft.vx=0;
	KidLeft.vy=0;
	KidLeft.direction=EAST;
	KidLeft.animdelay=80;
	KidLeft.animcount=0;
	KidLeft.rectsource.top=0;
	KidLeft.rectsource.bottom=KidLeft.height;
	KidLeft.rectsource.left=0;
	KidLeft.rectsource.right=KidLeft.width;
	KidLeft.visible=true;

	//set left projectile properties
	AirplaneLeft.x=0;
	AirplaneLeft.y=0;
	AirplaneLeft.width=32;
	AirplaneLeft.height=32;
	AirplaneLeft.vx=0;
	AirplaneLeft.vy=0;
	AirplaneLeft.direction=EAST;
	AirplaneLeft.visible=false;
	AirplaneLeft.alivetime=0;

	//set right projectile properties
	AirplaneRight.x=0;
	AirplaneRight.y=0;
	AirplaneRight.width=32;
	AirplaneRight.height=32;
	AirplaneRight.vx=0;
	AirplaneRight.vy=0;
	AirplaneRight.direction=WEST;
	AirplaneRight.visible=false;
	AirplaneRight.alivetime=0;

	//set desk properties
	for(int index=0;index<NUM_OF_DESKS;index++)
	{
		Desk[index].width=128;
		Desk[index].height=64;
		Desk[index].rectsource.left=0;
		Desk[index].rectsource.top=0;
		Desk[index].rectsource.right=Desk[index].rectsource.left+Desk[index].width;
		Desk[index].rectsource.bottom=Desk[index].rectsource.top+Desk[index].height;
		Desk[index].visible=true;
	}
	
	//set desk locations
	Desk[0].x=125;
	Desk[0].y=300;
	Desk[1].x=575;
	Desk[1].y=300;
	Desk[2].x=350;
	Desk[2].y=200;
	Desk[3].x=350;
	Desk[3].y=425;

	//set right and left player scoring images
	RightPlayerScores.width=280;
	RightPlayerScores.height=256;
	RightPlayerScores.x=290;
	RightPlayerScores.y=250;
	RightPlayerScores.visible=false;

	LeftPlayerScores.width=280;
	LeftPlayerScores.height=256;
	LeftPlayerScores.x=285;
	LeftPlayerScores.y=250;
	LeftPlayerScores.visible=false;

	//winning images
	RightPlayerWins.width=280;
	RightPlayerWins.height=256;
	RightPlayerWins.x=285;
	RightPlayerWins.y=250;
	RightPlayerWins.visible=false;

	LeftPlayerWins.width=280;
	LeftPlayerWins.height=256;
	LeftPlayerWins.x=285;
	LeftPlayerWins.y=250;
	LeftPlayerWins.visible=false;

	//set chalkboard images
	RightPlayerName.width=160;
	RightPlayerName.height=64;
	RightPlayerName.x=470;
	RightPlayerName.y=18;
	RightPlayerName.visible=true;

	LeftPlayerName.width=124;
	LeftPlayerName.height=72;
	LeftPlayerName.x=130;
	LeftPlayerName.y=13;
	LeftPlayerName.visible=true;

	RightPlayerChalkboardScore.width=24;
	RightPlayerChalkboardScore.height=60;
	RightPlayerChalkboardScore.x=660;
	RightPlayerChalkboardScore.y=24;
	RightPlayerChalkboardScore.rectsource.left=0;
	RightPlayerChalkboardScore.rectsource.top=0;
	RightPlayerChalkboardScore.rectsource.right=RightPlayerChalkboardScore.rectsource.left+RightPlayerChalkboardScore.width;
	RightPlayerChalkboardScore.rectsource.bottom=60;
	RightPlayerChalkboardScore.visible=true;

	LeftPlayerChalkboardScore.width=24;
	LeftPlayerChalkboardScore.height=60;
	LeftPlayerChalkboardScore.x=280;
	LeftPlayerChalkboardScore.y=24;
	LeftPlayerChalkboardScore.rectsource.left=0;
	LeftPlayerChalkboardScore.rectsource.top=0;
	LeftPlayerChalkboardScore.rectsource.right=LeftPlayerChalkboardScore.rectsource.left+LeftPlayerChalkboardScore.width;
	LeftPlayerChalkboardScore.rectsource.bottom=60;
	LeftPlayerChalkboardScore.visible=true;

	//set menu selector properties
	MenuSelector.width=54;
	MenuSelector.height=36;
	MenuSelector.visible=false;
	//set credit properties
	CreditMenu.width=800;
	CreditMenu.height=600;
	CreditMenu.visible=false;
	//set help properties
	HelpMenu.width=800;
	HelpMenu.height=600;
	HelpMenu.visible=false;

	//load sound into object
	//sound_test = LoadSound("bounce.wav");
	
	sound_hit1=LoadSound("hit1.wav");//doh
	sound_hit2=LoadSound("hit2.wav");//noob
	sound_hit3=LoadSound("hit3.wav");//pwnd
	//if sounds don't load, fail on game init
	if(sound_hit1==NULL || sound_hit2==NULL || sound_hit3==NULL)
	{return 0;}

	cerr<<"Game Specific Objects Initialized\n";

	//after objects initialized, go to game menu
	GameState=GAME_MENU;

	return 1;
}

//main game loop
void Game_Run(HWND hwnd)
{
	//update mouse movement
	Poll_Mouse();
	//update keyboard state
	Poll_Keyboard();

	//keyboard pressed variables
	//If a loop is gone through with the key up, it hasn't been pressed yet
	if(!Key_Down(DIK_RIGHT))
	{RightKeyPressed=false;}
	if(!Key_Down(DIK_LEFT))
	{LeftKeyPressed=false;}
	if(!Key_Down(DIK_UP))
	{UpKeyPressed=false;}
	if(!Key_Down(DIK_DOWN))
	{DownKeyPressed=false;}
	if(!Key_Down(DIK_A))
	{AKeyPressed=false;}
	if(!Key_Down(DIK_D))
	{DKeyPressed=false;}
	if(!Key_Down(DIK_W))
	{WKeyPressed=false;}
	if(!Key_Down(DIK_S))
	{SKeyPressed=false;}
	if(!Key_Down(DIK_F))
	{FKeyPressed=false;}
	if(!Key_Down(DIK_RCONTROL))
	{RcontrolKeyPressed=false;}
	if(!Key_Down(DIK_ESCAPE))
	{EscKeyPressed=false;}
	if(!Key_Down(DIK_RETURN))
	{EnterKeyPressed=false;}

	//main game state router
	switch(GameState)
	{
		//main menu
		case GAME_MENU:

			//quit game if escape key pressed
			if(Key_Down(DIK_ESCAPE) && !EscKeyPressed)
			{PostMessage(hwnd,WM_DESTROY,0,0);}

			//hide game images
			KidRight.visible=false;
			KidLeft.visible=false;

			RightPlayerName.visible=false;
			RightPlayerChalkboardScore.visible=false;

			LeftPlayerName.visible=false;
			LeftPlayerChalkboardScore.visible=false;

			for(int index=0;index<NUM_OF_DESKS;index++)
				Desk[index].visible=false;

			//display menu cursor
			MenuSelector.visible=true;

			//set menu cursor position based on current selection
			if(MenuSelectorPosition==1)
			{
				MenuSelector.x=525;
				MenuSelector.y=173;
			}
			else if(MenuSelectorPosition==2)
			{
				MenuSelector.x=526;
				MenuSelector.y=249;
			}
			else if(MenuSelectorPosition==3)
			{
				MenuSelector.x=575;
				MenuSelector.y=325;
			}
			else
			{
				MenuSelector.x=525;
				MenuSelector.y=399;
			}

			//change menu position based on keyboard input
			if(Key_Down(DIK_DOWN) && !DownKeyPressed)
			{
				DownKeyPressed=true;
				MenuSelectorPosition++;
				//if at bottom, stay at bottom option
				if(MenuSelectorPosition>4)
				{MenuSelectorPosition=4;}
			}
			if(Key_Down(DIK_UP) && !UpKeyPressed)
			{
				UpKeyPressed=true;
				MenuSelectorPosition--;
				//if at top, stay at top option
				if(MenuSelectorPosition<1)
				{MenuSelectorPosition=1;}
			}

			//if an option is selected...
			if(Key_Down(DIK_RETURN) && !EnterKeyPressed)
			{
				EnterKeyPressed=true;

				//play is selected
				if(MenuSelectorPosition==1)
				{
					//reset pieces to proper position
					GameState=GAME_RESET;

					//show game images
					KidRight.visible=true;
					KidLeft.visible=true;

					RightPlayerName.visible=true;
					RightPlayerChalkboardScore.visible=true;

					LeftPlayerName.visible=true;
					LeftPlayerChalkboardScore.visible=true;

					for(int index=0;index<NUM_OF_DESKS;index++)
						Desk[index].visible=true;

					//hide menu cursor
					MenuSelector.visible=false;

					//reset score
					LeftPlayerScore=0;
					RightPlayerScore=0;
				}
				else if(MenuSelectorPosition==2)
				{
					//switch to help
					GameState=GAME_MENU_HELP;
				}
				else if (MenuSelectorPosition==3)
				{
					//switch to credits
					GameState=GAME_MENU_CREDITS;
				}
				else
				{
					//Quit option selected, quit game
					PostMessage(hwnd,WM_DESTROY,0,0);
				}
			}

			break;

			//credits menu
		case GAME_MENU_CREDITS:
			
			//show credits background
			CreditMenu.visible=true;
			//set menu cursor to "back" option
			MenuSelector.x=525;
			MenuSelector.y=400;

			//return to menu if escape or return is hit
			if( Key_Down(DIK_ESCAPE) || (Key_Down(DIK_RETURN) && !EnterKeyPressed))
			{
				GameState=GAME_MENU;
				EscKeyPressed=true;
				CreditMenu.visible=false;
				EnterKeyPressed=true;
			}

			break;

			//Help menu selected
		case GAME_MENU_HELP:
			HelpMenu.visible=true;
			MenuSelector.x=505;
			MenuSelector.y=408;

			//return to menu if escape or return is hit
			if(Key_Down(DIK_ESCAPE) || (Key_Down(DIK_RETURN) && !EnterKeyPressed))
			{
				GameState=GAME_MENU;
				EscKeyPressed=true;
				HelpMenu.visible=false;
				EnterKeyPressed=true;
			}
            
			break;

		//game is currently running (play mode)
		case GAME_RUNNING:

			//Sound test, Removed at Release Version
			//if 'B' key is pressed, play sound
			//if(Key_Down(DIK_B))
			//{PlaySound(sound_test);}

			//check for player rotation
			//right player
			//if right key, rotate player clockwise
			if(Key_Down(DIK_RIGHT)&& !RightKeyPressed)
			{
				KidRight.direction++;
				if(KidRight.direction>SOUTHWEST)
				{KidRight.direction=WEST;}
				RightKeyPressed=true;
			}
			//if left key, rotate counter-clockwise
			if(Key_Down(DIK_LEFT)&& !LeftKeyPressed)
			{
				KidRight.direction--;
				if(KidRight.direction<WEST)
				{KidRight.direction=SOUTHWEST;}
				LeftKeyPressed=true;
			}
			//left Player
			if(Key_Down(DIK_D)&& !DKeyPressed)
			{
				KidLeft.direction++;
				if(KidLeft.direction>SOUTHWEST)
				{KidLeft.direction=WEST;}
				DKeyPressed=true;
			}
			if(Key_Down(DIK_A)&& !AKeyPressed)
			{
				KidLeft.direction--;
				if(KidLeft.direction<WEST)
				{KidLeft.direction=SOUTHWEST;}
				AKeyPressed=true;
			}

			//Set player velocity by direction
			SetVelocityByDirection(&KidRight,PLAYER_VELOCITY,PLAYER_VELOCITY);
			SetVelocityByDirection(&KidLeft,PLAYER_VELOCITY,PLAYER_VELOCITY);
				
			//check for forward and reverse motion
			//removed check for keypress because it needs to be a fluid motion, however I left the keypress bool functions intact in case we needed to reimplement
			//right player
			//forward motion
			if(Key_Down(DIK_UP))
			{
				KidRight.x+=KidRight.vx;
				KidRight.y+=KidRight.vy;
				
				//gets the current tick count to find out how many ticks have passed since last frame
				//player will only animate if it has been moving for a certain number of ticks,
				//set by animdelay
				if(GetTickCount()-KidRight.animcount>=KidRight.animdelay)
				{
					//if enough ticks have passed, change frame of animation
					KidRight.rectsource.top+=KidRight.height;
					KidRight.rectsource.bottom+=KidRight.height;

					//if all the frames are used, return to the top frame
					if(KidRight.rectsource.top>=KidRight.height*4)
					{
						KidRight.rectsource.top=0;
						KidRight.rectsource.bottom=KidRight.height;
					}
					
					//reset animation count information (this will be reset to current tick count later)
					KidRight.animcount=0;
				}
			
				//if animation already happened, reset counter to current tick count
				if(KidRight.animcount==0)
				{
					KidRight.animcount=GetTickCount();
				}

				UpKeyPressed=true;
			}
			//reverse motion
			if(Key_Down(DIK_DOWN))
			{
				KidRight.x-=KidRight.vx;
				KidRight.y-=KidRight.vy;

				if(GetTickCount()-KidRight.animcount>=KidRight.animdelay)
				{
					KidRight.rectsource.top+=KidRight.height;
					KidRight.rectsource.bottom+=KidRight.height;

					if(KidRight.rectsource.top>=KidRight.height*4)
					{
						KidRight.rectsource.top=0;
						KidRight.rectsource.bottom=KidRight.height;
					}
					
					KidRight.animcount=0;
				}
			
				if(KidRight.animcount==0)
				{KidRight.animcount=GetTickCount();}

				DownKeyPressed=true;
			}
			//left player
			if(Key_Down(DIK_W))
			{
				KidLeft.x+=KidLeft.vx;
				KidLeft.y+=KidLeft.vy;

				if(GetTickCount()-KidLeft.animcount>=KidLeft.animdelay)
				{
					KidLeft.rectsource.top+=KidLeft.height;
					KidLeft.rectsource.bottom+=KidLeft.height;

					if(KidLeft.rectsource.top>=KidLeft.height*4)
					{
						KidLeft.rectsource.top=0;
						KidLeft.rectsource.bottom=KidLeft.height;
					}
					
					KidLeft.animcount=0;
				}
				
				if(KidLeft.animcount==0)
				{KidLeft.animcount=GetTickCount();}

				WKeyPressed=true;
			}
			if(Key_Down(DIK_S))
			{
				KidLeft.x-=KidLeft.vx;
				KidLeft.y-=KidLeft.vy;

				if(GetTickCount()-KidLeft.animcount>=KidLeft.animdelay)
				{
					KidLeft.rectsource.top+=KidLeft.height;
					KidLeft.rectsource.bottom+=KidLeft.height;

					if(KidLeft.rectsource.top>=KidLeft.height*4)
					{
						KidLeft.rectsource.top=0;
						KidLeft.rectsource.bottom=KidLeft.height;
					}
					
					KidLeft.animcount=0;
				}
			
				if(KidLeft.animcount==0)
				{KidLeft.animcount=GetTickCount();}

				SKeyPressed=true;
			}

			//check for player firing
			//right player
			//doesn't fire if another airplane is already in flight
			if(Key_Down(DIK_RCONTROL) && !RcontrolKeyPressed && AirplaneRight.visible==false)
			{
				//set airplane to current kid position
				AirplaneRight.x=KidRight.x;
				AirplaneRight.y=KidRight.y;
				//make airplane visible
				AirplaneRight.visible=true;
				//airplane has been alive for 1 frame
				AirplaneRight.alivetime=1;
				//airplane takes off in same direction the kid is facing when fired
				AirplaneRight.direction=KidRight.direction;

				RcontrolKeyPressed=true;
			}
			//left player
			if(Key_Down(DIK_F) && !FKeyPressed && AirplaneLeft.visible==false)
			{
				AirplaneLeft.x=KidLeft.x;
				AirplaneLeft.y=KidLeft.y;
				AirplaneLeft.visible=true;
				AirplaneLeft.alivetime=1;
				AirplaneLeft.direction=KidLeft.direction;

				FKeyPressed=true;
			}
			
			//is left projectile still alive?
			if(AirplaneLeft.alivetime++<=AIRPLANE_LIFETIME)
			{
				//Set projectile velocity
				SetVelocityByDirection(&AirplaneLeft,AIRPLANE_X_VELOCITY,AIRPLANE_Y_VELOCITY);
				//move airplane
				AirplaneLeft.x+=AirplaneLeft.vx;
				AirplaneLeft.y+=AirplaneLeft.vy;
				//check for airplane collision with the wall
				CollisionAirplaneWithWalls(&AirplaneLeft);
				//check for airplane collision with the desks
				for(int index=0;index<NUM_OF_DESKS;index++)
				{
					CollisionAirplaneWithObject(&AirplaneLeft,&Desk[index]);
				}
			}
			//if alive too long, hide and reset alive counter
			else
			{
				AirplaneLeft.visible=false;
				AirplaneLeft.alivetime=0;
			}
			//is right projectile alive?
			if(AirplaneRight.alivetime++<=AIRPLANE_LIFETIME)
			{
				//Set projectile velocity
				SetVelocityByDirection(&AirplaneRight,AIRPLANE_X_VELOCITY,AIRPLANE_Y_VELOCITY);
				//move airplane
				AirplaneRight.x+=AirplaneRight.vx;
				AirplaneRight.y+=AirplaneRight.vy;

				//check for collision with walls
				CollisionAirplaneWithWalls(&AirplaneRight);
				//desk collisions
				for(int index=0;index<NUM_OF_DESKS;index++)
				{
					CollisionAirplaneWithObject(&AirplaneRight,&Desk[index]);
				}
			}
			else
			{
				AirplaneRight.visible=false;
				AirplaneRight.alivetime=0;
			}

			//Collision Detection
			//Right Player
			//with walls
			CollisionWithWalls(&KidRight);
			//with desks
			for(int index=0;index<NUM_OF_DESKS;index++)
			{CollisionPlayerWithObject(&KidRight,&Desk[index]);}
			//Left Player
			//with walls
			CollisionWithWalls(&KidLeft);
			//with desks
			for(int index=0;index<NUM_OF_DESKS;index++)
			{CollisionPlayerWithObject(&KidLeft,&Desk[index]);}

			//players with airplanes
			//right player left airplane
			//only test if the airplane is alive
			//if there is collision...left player scores
			if(CollisionPlayerWithAirplane(&KidRight,&AirplaneLeft) && AirplaneLeft.visible==true)
			{
				//hide airplane after score
				AirplaneLeft.visible=false;
				
				//increase left players score
				LeftPlayerScore++;

				//play a random sound, indicating one player's dominance over the other
				int randomsound=rand()%3;//seed random sound
				//play sound based on random
				if(randomsound==0)
					PlaySound(sound_hit1);
				else if(randomsound==1)
					PlaySound(sound_hit2);
				else if(randomsound==2)
					PlaySound(sound_hit3);
				
				//if the player has enough points to satisfy winning condition, they win
				if(LeftPlayerScore>=WINNING_CONDITION)
				{
					GameState=GAME_LEFT_PLAYER_WINS;
				}
				//if not, we go back to the game (after detouring through the score phase)
				else
				{
					GameState=GAME_LEFT_PLAYER_SCORE;
				}
			}
			//left player with right airplane
			if(CollisionPlayerWithAirplane(&KidLeft,&AirplaneRight) && AirplaneRight.visible==true)
			{
				AirplaneRight.visible=false;

				RightPlayerScore++;

				int randomsound=rand()%3;//seed random sound
				if(randomsound==0)
					PlaySound(sound_hit1);
				else if(randomsound==1)
					PlaySound(sound_hit2);
				else if(randomsound==2)
					PlaySound(sound_hit3);
				
				if(RightPlayerScore>=WINNING_CONDITION)
				{
					GameState=GAME_RIGHT_PLAYER_WINS;
				}
				else
				{
					GameState=GAME_RIGHT_PLAYER_SCORE;
				}

			}

			break;

		//if the right player scores
		case GAME_RIGHT_PLAYER_SCORE:
			
			//show scoring image
			RightPlayerScores.visible=true;

			//if they hit enter, hide the image and reset game positions
			if(Key_Down(DIK_RETURN))
			{
				RightPlayerScores.visible=false;
				GameState=GAME_RESET;
			}

			//if they hit escape, hide the scoring image (this was to fix a bug where hitting
			//escape after scoring would cause the menu to stay up)
			if(Key_Down(DIK_ESCAPE))
				RightPlayerScores.visible=false;

			break;
		
		//left player scores
		case GAME_LEFT_PLAYER_SCORE:
			LeftPlayerScores.visible=true;

			if(Key_Down(DIK_RETURN))
			{
				LeftPlayerScores.visible=false;
				GameState=GAME_RESET;
			}

			if(Key_Down(DIK_ESCAPE))
				LeftPlayerScores.visible=false;

			break;

		//right player wins the game!
		case GAME_RIGHT_PLAYER_WINS:
			
			//show winning image
			RightPlayerWins.visible=true;

			//if they hit enter, return to the main menu after hiding images
			if(Key_Down(DIK_RETURN))
			{
				RightPlayerWins.visible=false;
				EnterKeyPressed=true;
				GameState=GAME_MENU;
			}

			//bug fix to hide winning image if escape is hit
			if(Key_Down(DIK_ESCAPE))
			{
				RightPlayerWins.visible=false;
			}

			break;

		//Left player wins the game
		case GAME_LEFT_PLAYER_WINS:
			LeftPlayerWins.visible=true;

			if(Key_Down(DIK_RETURN))
			{
				LeftPlayerWins.visible=false;
				EnterKeyPressed=true;
				GameState=GAME_MENU;
			}

			if(Key_Down(DIK_ESCAPE))
			{
				LeftPlayerWins.visible=false;
			}

			break;

		//Player scores or New Game is selected, reset game positions 
		case GAME_RESET:
			//set Right Kid properties
			KidRight.x=750;
			KidRight.y=300;
			KidRight.vx=0;
			KidRight.vy=0;
			KidRight.direction=WEST;
			KidRight.animcount=0;
			KidRight.rectsource.top=0;
			KidRight.rectsource.bottom=KidRight.height;
			KidRight.rectsource.left=0;
			KidRight.rectsource.right=KidRight.width;

			//set Left Kid Properties
			KidLeft.x=50;
			KidLeft.y=300;
			KidLeft.vx=0;
			KidLeft.vy=0;
			KidLeft.direction=EAST;
			KidLeft.animcount=0;
			KidLeft.rectsource.top=0;
			KidLeft.rectsource.bottom=KidLeft.height;
			KidLeft.rectsource.left=0;
			KidLeft.rectsource.right=KidLeft.width;

			//set left projectile properties
			AirplaneLeft.x=0;
			AirplaneLeft.y=0;
			AirplaneLeft.vx=0;
			AirplaneLeft.vy=0;
			AirplaneLeft.direction=EAST;
			AirplaneLeft.visible=false;
			AirplaneLeft.alivetime=0;

			//set right projectile properties
			AirplaneRight.x=0;
			AirplaneRight.y=0;
			AirplaneRight.vx=0;
			AirplaneRight.vy=0;
			AirplaneRight.direction=WEST;
			AirplaneRight.visible=false;
			AirplaneRight.alivetime=0;
			
			//everything is reset, start playing the game
			GameState=GAME_RUNNING;

			break;

		default:
			//this option should never come up, if it does, there are problems
			cerr<<"Problem with Game State Switch\n";
			break;
	}

	//Rects for blitting onto backbuffer
	//these all take one frame of a tiled sprite to display
	KidRight.rectsource.left=KidRight.direction*KidRight.width;
	KidRight.rectsource.right=KidRight.rectsource.left+KidRight.width;

	KidLeft.rectsource.left=KidLeft.direction*KidLeft.width;
	KidLeft.rectsource.right=KidLeft.rectsource.left+KidLeft.width;

	AirplaneLeft.rectsource.left=AirplaneLeft.direction*AirplaneLeft.width;
	AirplaneLeft.rectsource.top=0;
	AirplaneLeft.rectsource.right=AirplaneLeft.rectsource.left+AirplaneLeft.width;
	AirplaneLeft.rectsource.bottom=AirplaneLeft.rectsource.top+AirplaneLeft.height;

	AirplaneRight.rectsource.left=AirplaneRight.direction*AirplaneRight.width;
	AirplaneRight.rectsource.top=0;
	AirplaneRight.rectsource.right=AirplaneRight.rectsource.left+AirplaneRight.width;
	AirplaneRight.rectsource.bottom=AirplaneRight.rectsource.top+AirplaneRight.height;

	RightPlayerChalkboardScore.rectsource.left=RightPlayerScore*24;
	RightPlayerChalkboardScore.rectsource.right=RightPlayerChalkboardScore.rectsource.left+24;

	LeftPlayerChalkboardScore.rectsource.left=LeftPlayerScore*24;
	LeftPlayerChalkboardScore.rectsource.right=LeftPlayerChalkboardScore.rectsource.left+24;

	//begin rendering scene
	if(d3ddev->BeginScene()==D3D_OK)
	{
		//clear back buffer
		d3ddev->Clear(0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,255),1.0f,0);

		//draw background to backbuffer
		d3ddev->StretchRect(background_surface,NULL,backbuffer,NULL,D3DTEXF_NONE);

		//if we are in the main menu, display the main menu surface over the background
		if(GameState==GAME_MENU)
		{
			d3ddev->StretchRect(mainmenu_surface,NULL,backbuffer,NULL,D3DTEXF_NONE);
		}

		//begin sprite drawing
		sprite_handler->Begin(D3DXSPRITE_ALPHABLEND);

		//position is a vector that stores sprite position in 3d coordinates
		//these if cases check to see if the sprite should be displayed, then draws it using
		//the spriter handler, with transparency

		//display the credits screen
		if(CreditMenu.visible==true)
		{
			position.x=0;
			position.y=0;
			//render
			sprite_handler->Draw(CreditMenu.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//display the help screen
		if(HelpMenu.visible==true)
		{
			position.x=0;
			position.y=0;
			//render
			sprite_handler->Draw(HelpMenu.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//display the menu cursor
		if(MenuSelector.visible==true)
		{
			position.x=(float)MenuSelector.x;
			position.y=(float)MenuSelector.y;
			//render
			sprite_handler->Draw(MenuSelector.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//displays the right player name on chalkboard
		if(RightPlayerName.visible==true)
		{
			position.x=(float)RightPlayerName.x;
			position.y=(float)RightPlayerName.y;
			//render
			sprite_handler->Draw(RightPlayerName.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		
		//displays left player name on chalkboard
		if(LeftPlayerName.visible==true)
		{
			position.x=(float)LeftPlayerName.x;
			position.y=(float)LeftPlayerName.y;
			//render
			sprite_handler->Draw(LeftPlayerName.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//displays the right player's score on chalkboard
		if(RightPlayerChalkboardScore.visible==true)
		{
			position.x=(float)RightPlayerChalkboardScore.x;
			position.y=(float)RightPlayerChalkboardScore.y;
			//render
			sprite_handler->Draw(RightPlayerChalkboardScore.texture,&RightPlayerChalkboardScore.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//displays the left player's score on the chalkboard
		if(LeftPlayerChalkboardScore.visible==true)
		{
			position.x=(float)LeftPlayerChalkboardScore.x;
			position.y=(float)LeftPlayerChalkboardScore.y;
			//render
			sprite_handler->Draw(LeftPlayerChalkboardScore.texture,&LeftPlayerChalkboardScore.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//Draw desks
		for(int index=0;index<NUM_OF_DESKS;index++)
		{
			if(Desk[index].visible==true)
			{
				position.x=(float)Desk[index].x;
				position.y=(float)Desk[index].y;

				sprite_handler->Draw(Desk[index].texture,&Desk[index].rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
		}

		//Draw Right Player
		if(KidRight.visible==true)
		{
			//vector holding sprite position (for sprite handler)
			position.x=(float)KidRight.x;
			position.y=(float)KidRight.y;
			//render sprite
			sprite_handler->Draw(KidRight.texture,&KidRight.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//Draw Left Player
		if(KidLeft.visible==true)
		{
			//update vector
			position.x=(float)KidLeft.x;
			position.y=(float)KidLeft.y;
			//render sprite
			sprite_handler->Draw(KidLeft.texture,&KidLeft.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//Draw Left Projectile
		if(AirplaneLeft.visible==true)
		{
			//update vector
			position.x=(float)AirplaneLeft.x;
			position.y=(float)AirplaneLeft.y;
			//render
			sprite_handler->Draw(AirplaneLeft.texture,&AirplaneLeft.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//Draw Right Projectile
		if(AirplaneRight.visible==true)
		{
			//update vector
			position.x=(float)AirplaneRight.x;
			position.y=(float)AirplaneRight.y;
			//render
			sprite_handler->Draw(AirplaneRight.texture,&AirplaneRight.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//draw left and right player scoring images
		if(RightPlayerScores.visible==true)
		{
			position.x=(float)RightPlayerScores.x;
			position.y=(float)RightPlayerScores.y;
			//render
			sprite_handler->Draw(RightPlayerScores.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		if(LeftPlayerScores.visible==true)
		{
			position.x=(float)LeftPlayerScores.x;
			position.y=(float)LeftPlayerScores.y;
			//render
			sprite_handler->Draw(LeftPlayerScores.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//draw winning images
		if(RightPlayerWins.visible==true)
		{
			position.x=(float)RightPlayerWins.x;
			position.y=(float)RightPlayerWins.y;
			//render
			sprite_handler->Draw(RightPlayerWins.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		if(LeftPlayerWins.visible==true)
		{
			position.x=(float)LeftPlayerWins.x;
			position.y=(float)LeftPlayerWins.y;
			//render
			sprite_handler->Draw(LeftPlayerWins.texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}

		//end sprite drawing
		sprite_handler->End();

		d3ddev->EndScene();
	}//stop rendering scene

	//present backbuffer onto primary
	d3ddev->Present(NULL,NULL,NULL,NULL);

	//check for escape key to return to the main menu
	if(Key_Down(DIK_ESCAPE) && !EscKeyPressed)
	{
		EscKeyPressed=true;
		GameState=GAME_MENU;
	}
		

	//synchronize each frame
	while(GetTickCount()-start<15)
	{}
	start=GetTickCount();
}

void Game_End (HWND hwnd)
{
	//Release Surfaces
	background_surface->Release();
	mainmenu_surface->Release();
	cerr<<"Surfaces Released\n";

	//Release Textures
	KidRight.texture->Release();
	KidLeft.texture->Release();
	AirplaneLeft.texture->Release();
	AirplaneRight.texture->Release();
	for(int index=0;index<NUM_OF_DESKS;index++)
	{
		Desk[index].texture->Release();
	}
	RightPlayerScores.texture->Release();
	LeftPlayerScores.texture->Release();
	RightPlayerName.texture->Release();
	LeftPlayerName.texture->Release();
	RightPlayerChalkboardScore.texture->Release();
	LeftPlayerChalkboardScore.texture->Release();
	MenuSelector.texture->Release();
	RightPlayerWins.texture->Release();
	LeftPlayerWins.texture->Release();

	HelpMenu.texture->Release();
	CreditMenu.texture->Release();

	cerr<<"Textures Released\n";

	//Delete Sounds
	//if(sound_test !=NULL)//testing purposes
	//{delete sound_test;}
	if(sound_hit1 !=NULL)
	{delete sound_hit1;}
	if(sound_hit2 !=NULL)
	{delete sound_hit2;}
	if(sound_hit3 !=NULL)
	{delete sound_hit3;}

	cerr<<"Sounds Deleted\n";

	cerr<<"\nGame shut down!\n";

}


//ADDITIONAL FUNCTIONS/////////////////////////////

//sets the velocity for the sprite based on what direction it's facing
void SetVelocityByDirection(SPRITE *sprite, int vx,int vy)
{
	//get sprite direction
	switch (sprite->direction)
	{
		//facing west, move left
		case WEST:
			sprite->vx=-vx;
			sprite->vy=0;
			break;
		case NORTHWEST:
			sprite->vx=-vx;
			sprite->vy=-vy;
			break;
		case NORTH:
			sprite->vx=0;
			sprite->vy=-vy;
			break;
		case NORTHEAST:
			sprite->vx=vx;
			sprite->vy=-vy;
			break;
		case EAST:
			sprite->vx=vx;
			sprite->vy=0;
			break;
		case SOUTHEAST:
			sprite->vx=vx;
			sprite->vy=vy;
			break;
		case SOUTH:
			sprite->vx=0;
			sprite->vy=vy;
			break;
		case SOUTHWEST:
			sprite->vx=-vx;
			sprite->vy=vy;
			break;

		//just in case
		default:
			sprite->vx=1;
			sprite->vy=1;
			break;
	}
}//end setvelocitybydirection()

//collision detection against walls
void CollisionWithWalls(SPRITE *sprite)
{
	//if sprite hits left wall
	if(sprite->x<=0)
	{
		//set sprite position to left wall
		sprite->x=0;
		//reverse x velocity
		sprite->vx=-sprite->vx;
	}

	//right wall
	if(sprite->x+sprite->width>=SCREEN_WIDTH)
	{
		sprite->x=SCREEN_WIDTH-sprite->width;
		sprite->vx=-sprite->vx;
	}

	//top wall
	if(sprite->y<=60)//80 not 0 for chalkboard collision
	{
		sprite->y=60;
		sprite->vy=-sprite->vy;
	}

	//bottom wall
	if(sprite->y+sprite->height>=SCREEN_HEIGHT)
	{
		sprite->y=SCREEN_HEIGHT-sprite->height;
		sprite->vy=-sprite->vy;
	}

}//end CollisionWithWalls

//airplane with walls collision detection
void CollisionAirplaneWithWalls(SPRITE *airplane)
{
	//left wall
	if(airplane->x<=0)
	{
		airplane->x=0;

		//bounce based on what direction airplane was flying
		if(airplane->direction==WEST)
			airplane->direction=SOUTHEAST;
		else if(airplane->direction==NORTHWEST)
			airplane->direction=NORTHEAST;
		else if (airplane->direction==SOUTHWEST)
			airplane->direction=SOUTHEAST;
	}

	//right wall
	if(airplane->x>=SCREEN_WIDTH-airplane->width)
	{
		airplane->x=SCREEN_WIDTH-airplane->width;

		if(airplane->direction==EAST)
			airplane->direction=NORTHWEST;
		else if(airplane->direction==NORTHEAST)
			airplane->direction=NORTHWEST;
		else if(airplane->direction==SOUTHEAST)
			airplane->direction=SOUTHWEST;
	}

	//if airplane hits top wall
	if(airplane->y<=40)//40 not 0 for mid-chalkboard collision
	{
		airplane->y=40;

		if(airplane->direction==NORTH)
			airplane->direction=SOUTHWEST;
		else if(airplane->direction==NORTHWEST)
			airplane->direction=SOUTHWEST;
		else if(airplane->direction==NORTHEAST)
			airplane->direction=SOUTHEAST;
	}

	//bottom wall
	if(airplane->y>=SCREEN_HEIGHT-airplane->height)
	{
		airplane->y=SCREEN_HEIGHT-airplane->height;

		if(airplane->direction==SOUTH)
			airplane->direction=NORTHEAST;
		else if(airplane->direction==SOUTHEAST)
			airplane->direction=NORTHEAST;
		else if(airplane->direction==SOUTHWEST)
			airplane->direction=NORTHWEST;
	}

}//end collisionairplanewithwalls()

//mainly for player against desk collision
void CollisionPlayerWithObject(SPRITE *sprite,SPRITE *object)
{

	//if sprite intersects left wall of object
	if(sprite->x+sprite->width-KID_HORIZONTAL_BOUNDING_REDUCTION >= object->x && sprite->x+KID_HORIZONTAL_BOUNDING_REDUCTION<object->x)
	{
		//if sprite is in the right height of the object
		if(sprite->y+sprite->height > object->y && sprite->y < object->y+object->height)
		{
			//subtract overlap from position
			//sprite->x -= (sprite->x+sprite->width) - object->x;
			sprite->x=object->x-sprite->width+KID_HORIZONTAL_BOUNDING_REDUCTION;
		}
	}

	//right wall
	if(sprite->x+KID_HORIZONTAL_BOUNDING_REDUCTION<=object->x+object->width && sprite->x+sprite->width-KID_HORIZONTAL_BOUNDING_REDUCTION > object->x+object->width)
	{
		if(sprite->y+sprite->height > object->y && sprite->y < object->y+object->height)
		{
			//sprite->x += (object->x+object->width) - sprite->x;
			sprite->x=object->x+object->width-KID_HORIZONTAL_BOUNDING_REDUCTION;
		}
	}

	//top wall
	if(sprite->y+sprite->height >= object->y && sprite->y<object->y)
	{
		if(sprite->x+sprite->width-KID_HORIZONTAL_BOUNDING_REDUCTION > object->x && sprite->x+KID_HORIZONTAL_BOUNDING_REDUCTION < object->x+object->width)
		{
				sprite->y -= (sprite->y+sprite->height) - object->y;
		}
	}

	//bottom wall
	if(sprite->y <= object->y+object->height && sprite->y+sprite->height > object->y+object->height)
	{
		if(sprite->x+sprite->width-KID_HORIZONTAL_BOUNDING_REDUCTION > object->x && sprite->x+KID_HORIZONTAL_BOUNDING_REDUCTION < object->x+object->width)
		{
			sprite->y += (object->y+object->height) - sprite->y;	
		}
	}
}//end collisionplayerwithobject()

//airplanes with desks
void CollisionAirplaneWithObject(SPRITE *airplane, SPRITE *object)
{
	//right wall
	if(airplane->x <= object->x+object->width && airplane->x+airplane->width > object->x+object->width)
	{
		if(airplane->y+airplane->width > object->y && airplane->y < object->y+object->height)
		{
			airplane->x=object->x+object->width;

			//change flight direction based on initial collision direction
			if(airplane->direction==WEST)
				airplane->direction=SOUTHEAST;
			else if(airplane->direction==NORTHWEST)
				airplane->direction=NORTHEAST;
			else if (airplane->direction==SOUTHWEST)
				airplane->direction=SOUTHEAST;
		}
	}
	//left wall
	if(airplane->x+airplane->width >= object->x && airplane->x < object->x)
	{
		if(airplane->y+airplane->width > object->y && airplane->y < object->y+object->height)
		{
			airplane->x=object->x-airplane->width;

			if(airplane->direction==EAST)
				airplane->direction=NORTHWEST;
			else if(airplane->direction==NORTHEAST)
				airplane->direction=NORTHWEST;
			else if(airplane->direction==SOUTHEAST)
				airplane->direction=SOUTHWEST;
		}
	}
	
	//top wall
	if(airplane->y+airplane->height >= object->y && airplane->y  < object->y)
	{
		if(airplane->x+airplane->width > object->x && airplane->x < object->x+object->width)
		{
			airplane->y=object->y-airplane->height;

			if(airplane->direction==SOUTH)
				airplane->direction=NORTHEAST;
			else if(airplane->direction==SOUTHEAST)
				airplane->direction=NORTHEAST;
			else if(airplane->direction==SOUTHWEST)
				airplane->direction=NORTHWEST;
		}
	}

	//bottom wall
	if(airplane->y <= object->y+object->height && airplane->y+airplane->height > object->y+object->height)
	{
		if(airplane->x+airplane->width > object->x && airplane->x < object->x+object->width)
		{
			airplane->y=object->y+object->height;
			
			if(airplane->direction==NORTH)
				airplane->direction=SOUTHWEST;
			else if(airplane->direction==NORTHWEST)
				airplane->direction=SOUTHWEST;
			else if(airplane->direction==NORTHEAST)
				airplane->direction=SOUTHEAST;	
		}
	}

}//end collisionairplanewithobject()

//player with an airplane collision (could you not figure that out?)
int CollisionPlayerWithAirplane(SPRITE *player, SPRITE *airplane)
{
	//creates rectangles of positions, plus a dummy rect for god knows what reason
	RECT rPlayer,rAirplane,rDummy;

	//set rect's based on position
	//subtract a constant amount to reduce the bounding box (tighter collisions)
	rPlayer.left=player->x+KID_HORIZONTAL_BOUNDING_REDUCTION;
	rPlayer.right=player->x+player->width-KID_HORIZONTAL_BOUNDING_REDUCTION;
	rPlayer.top=player->y+KID_VERTICAL_BOUNDING_REDUCTION;
	rPlayer.bottom=player->y+player->height-KID_VERTICAL_BOUNDING_REDUCTION;

	rAirplane.left=airplane->x;
	rAirplane.right=rAirplane.left+airplane->width;
	rAirplane.top=airplane->y;
	rAirplane.bottom=rAirplane.top+airplane->height;

	//if they intersect at all, collision exists
	if(IntersectRect(&rDummy,&rPlayer,&rAirplane))
		return 1;

	return 0;
}//end collisionplayerwithairplane()